
% LOAD IN tilt RESULTS and MEX %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear; close all; 

mex Ctransform.c; 
mex Clearn.c;   
mex Cfind.c;   
mex Ccopy.c; 
mex Cmax.c; 

% CONSTANTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
scale = .7;  % scale of Gabors, length = 17 pixels
norient = 15;  % number of orientations
Upperbound = 16.; % saturation value
epsilon = .1;  % allowed correlation between selected Gabors
sub = 1;    % subsampling rate for Gabor
Lrange = 3; % allowed shifting in location (times sub pixels)
Orange = 1; % allowed shifting in orientation
SHUTUP = 1.; % SHUTUP is to neglect the positions with low average responses               
binnum = 50; % number of bins 
nstep = 3;  % number of iterations
Totalsketch = 40;  % allowed number of selected Gabors   
sub1 = sub; Lrange1 = Lrange; 
sx0 = 100; sy0 = 100;  
fold = 1.; 

% FILTER %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[allfilter, allsymbol] = makefilter(scale, norient);  % generate Gabor filters 
h = (size(allfilter{1}, 1)-1)/2;  % half size of Gabor
C = corr(allfilter, epsilon);    % generate the inhibition maps

% TRAINING IMAGES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Folder = 'train';  % image input folder
Iname = dir([Folder '\*.jpg']);
nimage = size(Iname, 1); sxytotal = 0.; 
for (i = 1 : nimage)
    J{i} = imresize(double(rgb2gray(imread([Folder '\' Iname(i).name]))), fold, 'nearest');
    [sx1, sy1] = size(J{i}); 
    sxytotal = sxytotal + sx1*sy1; 
    Sx(i) = sx1; Sy(i) = sy1; % sizes of images
end
ax = sx0; ay = sy0; 
Sx0 = zeros(1, nimage) + sx0; 
Sy0 = zeros(1, nimage) + sy0; 


totalScale0 = 3;  oj = 2; % total number of scales
totalStretch = 1;
totalScale = totalScale0*totalStretch; 
ax = sx0; ay = sy0; 
Sxx = zeros(nimage, totalScale); Sxa = zeros(1, totalScale*nimage); 
Syy = zeros(nimage, totalScale); Sya = zeros(1, totalScale*nimage); 
mscale = zeros(nimage, totalScale);  % log-likelihood score over scales
JJ = cell(1, totalScale*nimage); 
tss1 = cell(1, totalScale*nimage); 
sym1 = cell(1, totalScale*nimage); 

for (i = 1:nimage)
 for (j0=1:totalStretch)
  for(j1=1:totalScale0)
    j = j0+(j1-1)*totalStretch; 
    scaleF = .9+(j1-1)*.1; 
    stretchF = 1. + (j0-1)*.4;  
    [sx00 sy00] = size(J{i}); 
    JJ{(i-1)*totalScale+j} = imresize(J{i},[round(sx00*scaleF) round(sy00*scaleF*stretchF)], 'nearest');  % scaled images
    [sx1, sy1] = size(JJ{(i-1)*totalScale+j}); 
    Sxx(i, j) = sx1; Syy(i, j) = sy1; % sizes of images
    Sxa((i-1)*totalScale+j) = sx1; Sya((i-1)*totalScale+j) = sy1; 
    tss1{(i-1)*totalScale+j} = zeros(sx1+ax, sy1+ay);  % log-likelihood score map for testing images
    sym1{(i-1)*totalScale+j} = zeros(sx1, sy1);  % symbolic sketch on testing image
  end
 end
end

disp(['start filtering training images at all scales']);
tic
allfiltered1 = applyfilterfft(JJ, allfilter);  % filter training images
disp(['filtering time: ' num2str(toc) ' seconds']);

Ctransform(totalScale*nimage, norient, allfiltered1, Sxa, Sya, Upperbound);
allmaxed = cell(totalScale*nimage, norient);
for (i = 1:totalScale*nimage)
    for (o = 1:norient)
        allmaxed{i, o} = allfiltered1{i, o} + 0.; 
    end
end
Cmax(totalScale*nimage, norient, allmaxed, Sxa, Sya, Lrange1, Orange, sub1);

allfiltered = cell(nimage, norient); 
for (i = 1 : nimage)
    for (o = 1 : norient)
        allfiltered{i, o} = zeros(sx0, sy0); 
    end
end
allfilter0 = cell(1, norient); 
for (o = 1 : norient)
      allfiltered0{1, o} = zeros(sx0, sy0); 
end

% VARIABLES FOR MEX-C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Mi = zeros(1, Totalsketch);  % orientation of selected Gabor
Mx = zeros(1, Totalsketch);  % position of selected Gabor
My = zeros(1, Totalsketch); 
Mm = zeros(1, Totalsketch);  % lambda
sym = zeros(sx0, sy0);  % symbolic plot of selected Gabor
Asym = cell(1, nimage);   % symbolic plot for each image with active Gabor
for (i = 1 : nimage)
    Asym{i} = zeros(sx0, sy0);  
end
tss = zeros(1, nimage); 


% START FROM SINGLE IMAGE LEARNING %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
scanf = 1./3.; ii = 0; 
iitotal = floor(sxytotal/sx0/sy0/scanf/scanf*1.1); 
thisi = zeros(1, iitotal); 
thisx = zeros(1, iitotal); thisy = zeros(1, iitotal); 
thistss = zeros(1, iitotal);
showon = 0.; 

for (this = 1 : nimage)
Jt = J{this}; 
[sxt syt] = size(Jt); 
for (xt0 = 0 : floor((sxt-sx0-1)/(sx0*scanf)))
    for (yt0 = 0: floor((syt-sy0-1)/(sy0*scanf)))
        xt = floor(xt0*sx0*scanf); 
        yt = floor(yt0*sy0*scanf); 
        ii = ii+1; 
        thisi(ii) = this; 
        thisx(ii) = xt;
        thisy(ii) = yt;
        [this xt yt]
     ABlearnthis;       
    end
end
end

showon = 1.; 
[thistsssort ind] = sort(thistss, 'descend'); 
K = 3; 
for (k = 1:K)
    ib = ind(k); 
    this = thisi(ib); 
    Jt = J{this}; 
    [sxt syt] = size(Jt); 
    xt = thisx(ib); 
    yt = thisy(ib); 
    ABlearnthis; 
end
for (k=1:K)
disp(['<IMG SRC="ex5d_wordcarL_sym' num2str(k) '.gif"height=100>']); 
for(i=1:nimage)
      disp(['<IMG SRC="ex5d_wordcarL' num2str(k) '_' num2str(i) '.gif"height=100>']);  % overlaid sketch 
end
disp('<br><hr>'); 
end
